﻿Type=Class
Version=3.5
@EndOfDesignText@
'version 2.17 mHasFocus
'Class module
Sub Class_Globals
	Private ET As EditText
	Private lv As ListView
	Private mCallback As Object
	Private mEventName As String
	
	Private mEditTextColor As Int
	Private mItemColor As Int
	Private mLVMaxHeight As Int
	
	Private DB As SQL
	Private DBName As String
	Private TBName As String
	
	' Constants
	Public SELECTION_MODE_ALL As Int			: SELECTION_MODE_ALL = 1
	Public SELECTION_MODE_SUBSTRING As Int	: SELECTION_MODE_SUBSTRING = 2
	Public SELECTION_MODE_PREFIX As Int		: SELECTION_MODE_PREFIX = 3

	Private mSelectionMode As Int
	Private mCaseSensitive As Boolean = False
	
	Private QueryX As String
	
	Private IME1 As IME
	IME1.Initialize("Ime")
	Private KeyboardTop As Int
	Private KeyboardVisible As Boolean
	Private mHasFocus As Boolean
	
	Private ETInitialTop As Int
	Private CurrentETTop As Int
	
End Sub

'Initializes the object and sets the module and sub that will handle the ItemClick event
' ATTENTION: "Name" must be unique and should be set only once, because it will be the name
' of an "internal" database table. If you change it, then you should delete old table(s)
' from the database "manually".
Public Sub Initialize (Callback As Object, EventName As String, Name As String)
	TBName = Name
	
	ET.Initialize("et")
	'Remove the suggestions bar
	ET.InputType = Bit.OR(ET.INPUT_TYPE_TEXT, 0x00080000)
	lv.Initialize("lv")
	lv.SingleLineLayout.ItemHeight = 42dip
	lv.SingleLineLayout.Label.TextSize = 14
	lv.SingleLineLayout.Label.Color = Colors.Black
	lv.SingleLineLayout.Label.TextColor = Colors.White
	lv.Visible = False
	mCallback = Callback
	mEventName = EventName
	
	InitDB

	mSelectionMode = SELECTION_MODE_PREFIX
	
	IME1.AddHeightChangedEvent
End Sub

#Region  + Properties

#Region  CaseSensitive 
Public Sub setCaseSensitive(CaseSensitive As Boolean)
	If CaseSensitive <> mCaseSensitive Then
		mCaseSensitive = CaseSensitive
		If mCaseSensitive Then
			DB.ExecNonQuery("PRAGMA case_sensitive_like=ON")
		Else
			DB.ExecNonQuery("PRAGMA case_sensitive_like=OFF")
		End If
		Private Text As String = ET.Text
		ET.Text = ""
		ET.Text = Text
	End If
End Sub
Public Sub getCaseSensitive As Boolean
	Return mCaseSensitive
End Sub
#End Region

#Region  + Layout 

#Region  Left 
Public Sub setLeft(Left As Int)
'	mLeft = Left
'	SetLayout
	ET.Left = Left
	lv.Left = Left
End Sub
Public Sub getLeft As Int
	Return ET.Left
End Sub
#End Region

#Region  Top 
Public Sub setTop(Top As Int)
'	mTop = Top
'	SetLayout
	ET.Top = Top
	lv.Top = Top + ET.Height
End Sub
Public Sub getTop As Int
	Return ET.Top
End Sub
#End Region

#Region  Width 
Public Sub setWidth(Width As Int)
'	mWidth = Width
'	SetLayout
	ET.Width = Width
	lv.Width = Width
End Sub
Public Sub getWidth As Int
	Return ET.Width
End Sub
#End Region

#Region  Height 
Public Sub setHeight(Height As Int)
'	mHeight = Height
'	SetLayout
	ET.Height = Height
End Sub
Public Sub getHeight As Int
	Return ET.Height
End Sub
#End Region

Public Sub SetLayout(Left As Int, Top As Int, Width As Int, Height As Int)
	ET.SetLayout(Left, Top, Width, Height)
	lv.Left = Left
	lv.Top = ET.Top + ET.Height
	lv.Width = Width
End Sub

#End Region

#Region  Selection mode 
Public Sub setSelectionMode(SelectionMode As Int)
	mSelectionMode = SelectionMode
	Private Text As String = ET.Text
	ET.Text = ""
	ET.Text = Text
End Sub
Public Sub getSelectionMode As Int
	Return mSelectionMode
End Sub
#End Region

#Region  + Single line layout 

#Region  Item Height
Public Sub setItemHeight(ItemHeight As Int)
	lv.SingleLineLayout.Label.Height = ItemHeight
End Sub
Public Sub getItemHeight As Int
	Return lv.SingleLineLayout.Label.Height
End Sub
#End Region

#Region  Item text size
Public Sub setItemTextSize(ItemTextSize As Float)
	lv.SingleLineLayout.Label.TextSize = ItemTextSize
End Sub
Public Sub getItemTextSize As Float
	Return lv.SingleLineLayout.Label.TextSize
End Sub
#End Region

#Region  Item text Color
Public Sub setItemTextColor(ItemTextColor As Int)
	lv.SingleLineLayout.Label.TextColor = ItemTextColor
End Sub
Public Sub getItemTextColor As Int
	Return lv.SingleLineLayout.Label.TextColor
End Sub
#End Region

#Region  Item Color
Public Sub setItemColor(ItemColor As Int)
	mItemColor = ItemColor
	lv.SingleLineLayout.Label.Color = mItemColor
End Sub
Public Sub getItemColor As Int
	Return mItemColor
End Sub
#End Region

#End Region

#Region  + EditText Text properties 

#Region  Text 
Public Sub setText(Text As String)
	ET.Text = Text
End Sub
Public Sub getText As String
	Return ET.Text
End Sub
#End Region

#Region  Text Color 
Public Sub setTextColor(TextColor As Int)
	ET.TextColor = TextColor
End Sub
Public Sub getTextColor As Int
	Return ET.TextColor
End Sub
#End Region

#Region  Color 
Public Sub setColor(Color As Int)
	mEditTextColor = Color
	ET.Color = mEditTextColor
End Sub
Public Sub getColor As Int
	Return mEditTextColor
End Sub
#End Region

#Region  Text Size 
Public Sub setTextSize(TextSize As Float)
	ET.TextSize = TextSize
End Sub
Public Sub getTextSize As Float
	Return ET.TextSize
End Sub
#End Region

#Region  Text Typeface 
Public Sub setTypeface(NewTypeface As Typeface)
	ET.Typeface = NewTypeface
End Sub
Public Sub getTypeface As Typeface
	Return ET.Typeface
End Sub
#End Region

#End Region

#Region  Hint 
Public Sub setHint(Hint As String)
	ET.Hint = Hint
End Sub
Public Sub getHint As String
	Return ET.Hint
End Sub
#End Region

#Region  Hint Color
Public Sub setHintColor(HintColor As Int)
	ET.HintColor = HintColor
End Sub
Public Sub getHintColor As Int
	Return ET.HintColor
End Sub
#End Region

#End Region

#Region  Public methods 

'Adds the view to the parent. The parent can be an Activity or Panel.
' MaxListViewHeight should be set to 0 for unlimited.
Public Sub AddToParent(Parent As Panel, Left As Int, Top As Int, Width As Int, Height As Int, MaxListViewHeight As Int)
	' EditText
	Parent.AddView(ET, Left, Top, Width, Height)

	Private ListViewHeight As Int = 168dip
	Private ListViewTop As Int = Top + Height

	If ListViewTop + ListViewHeight > 100%y Then
		ListViewTop = Top - ListViewHeight
	End If
	Parent.AddView(lv, Left, ListViewTop, Width, 168dip)
	
	mLVMaxHeight = MaxListViewHeight
	If mLVMaxHeight = 0 Then
		mLVMaxHeight = 100%y - ListViewTop
	End If
	
	ETInitialTop = ET.Top
	CurrentETTop = ETInitialTop
	
End Sub

Public Sub AddItemList(ItemList As List) As Boolean
	Dim Res As Boolean
	
	Dim Query As String = "INSERT OR REPLACE INTO " & TBName & " (Item) VALUES (?)"

	DB.BeginTransaction
	Try
		For r = 0 To ItemList.Size - 1
			DB.ExecNonQuery2(Query, Array As Object(ItemList.get(r)))
		Next
		DB.TransactionSuccessful
		Res = True
	Catch
		Res = False
	End Try
	DB.EndTransaction
	
	Return Res
End Sub

#Region  + DB commands 

Public Sub GetItemList(Text As String, SelectionMode As Int) As List
	Private Res As List

	QueryX = "SELECT * FROM " & TBName
	If SelectionMode = SELECTION_MODE_ALL Then
		' Nothing to add.
	Else If SelectionMode = SELECTION_MODE_PREFIX Then
		QueryX = QueryX & " WHERE Item LIKE '" & Text & "%'"
	Else If SelectionMode = SELECTION_MODE_SUBSTRING Then
		QueryX = QueryX & " WHERE Item LIKE '%" & Text & "%'"
	Else
		' invalid selection type.
	End If
		
	Private Curs As Cursor
	Curs = DB.ExecQuery(QueryX)
	
	If Curs.RowCount > 0 Then
		Res.Initialize
		Curs.Position = 0
		For r = 0 To Curs.RowCount - 1
			Res.Add(Curs.GetString("Item"))
			Curs.Position = Curs.Position + 1
		Next
	End If
	
	Return Res
	
End Sub

Public Sub AddItem(Text As String)
	QueryX = "INSERT OR REPLACE INTO " & TBName & " (Item) VALUES ('" & Text & "')"
	DB.ExecNonQuery(QueryX)
End Sub

Public Sub DeleteAllItems
	QueryX = "DELETE FROM " & TBName
	DB.ExecNonQuery(QueryX)
End Sub

Public Sub DeleteItem(ItemText As String)
	QueryX = "DELETE FROM " & TBName & " WHERE Item = '" & ItemText & "'"
	DB.ExecNonQuery(QueryX)
End Sub

Public Sub DBCLose
	DB.Close
End Sub

#End Region

#End Region

#Region  Private methods 

#Region  + DB commands 

Private Sub InitDB
	
	Private DBDir As String
	If File.ExternalWritable Then
		DBDir = File.DirDefaultExternal
	Else
		DBDir = File.DirInternal
	End If
	
	' Create database name same as package name plus .db extension
	DBName = GetPackageName & ".db"

	' Open database or create if it not exists
	DB.Initialize(DBDir, DBName, True)

	' Create table
	QueryX = "CREATE TABLE IF NOT EXISTS " & TBName & " (Item TEXT)"
	
	' Create index
	DB.ExecNonQuery(QueryX)
	QueryX = "CREATE UNIQUE INDEX IF NOT EXISTS ItemIdx ON " & TBName & " (Item)"
	DB.ExecNonQuery(QueryX)
	
End Sub

#End Region

Private Sub GetPackageName As String
  Dim ActMan As JavaObject
  Dim R As Reflector
  R.Target=R.GetContext
  'Get Activity Manager Object
  ActMan = R.RunMethod2("getSystemService","activity","java.lang.String")
  Dim TaskInfo As JavaObject = ActMan.RunMethod("getRunningTasks",Array As Object(1))
  Dim CompInfo As JavaObject = TaskInfo.RunMethodJO("get",Array As Object(0)).GetField("topActivity")
  Dim PackageName As String = CompInfo.RunMethod("getPackageName",Null)
  Return PackageName
End Sub

#End Region

#Region  Views events 

Private Sub lv_ItemClick (Position As Int, Value As Object)
	ET.Text = Value
	ET.SelectionStart = ET.Text.Length
'	IME.HideKeyboard
	lv.Visible = False
	If SubExists(mCallback, mEventName & "_ItemClick") Then
		CallSub2(mCallback, mEventName & "_ItemClick", Value)
	End If
End Sub

Private Sub et_TextChanged (Old As String, New As String)
	Private lst As List = GetItemList(New, mSelectionMode)
	lv.Clear
	If lst.IsInitialized Then
		For i = 0 To lst.Size - 1
			lv.AddSingleLine(lst.get(i))
		Next
		ResizeListViewHeight
	End If
	
	' After this statement, et "returns" to its initial position,
	' covered by keyboard!
	lv.Visible = True
	ResizeForKeyboard
'	IME1.ShowKeyboard(et)
End Sub

Private Sub et_FocusChanged (HasFocus As Boolean)
Log("* et_FocusChanged event")
	mHasFocus = HasFocus
Log("mHasFocus = " & mHasFocus)
	If Not(HasFocus) Then
		lv.Visible = False
	End If
End Sub

Sub Ime_HeightChanged(NewHeight As Int, OldHeight As Int)
Log("* Ime_HeightChanged event")
Log("mHasFocus into Ime_HeightChanged = " & mHasFocus)

	Dim KeyboardOldVisibility As Boolean = KeyboardVisible

	' NewHeight is like keyboard.top
	KeyboardTop = NewHeight
	
	KeyboardVisible = (NewHeight < OldHeight)
	
	' If keyboard changes its visibility,
	' set ET and LV at them right position.
	If mHasFocus AND KeyboardVisible <> KeyboardOldVisibility Then
		ResizeForKeyboard
	End If
	
End Sub

#End Region

Private Sub ResizeListViewHeight

	Dim MinNumOfVisibleItems As Int = 4
	
	Dim AvailableHeight As Int
	If KeyboardVisible Then
		AvailableHeight = KeyboardTop
	Else
		AvailableHeight = 100%y
	End If
	
	' RequiredHeight is the height required to display all items.
	Dim RequiredHeight As Int = lv.Size * lv.SingleLineLayout.ItemHeight
	RequiredHeight = Min(mLVMaxHeight, RequiredHeight)
	
	Dim etBottom As Int = CurrentETTop + ET.Height
	Dim LVTop As Int = etBottom
	
	' Tries to set the lv beneath the et, with the required height
	' or less, if the lv contains few items.
	If LVTop + RequiredHeight <= AvailableHeight Then
		lv.Height = Min(RequiredHeight, lv.Size * lv.SingleLineLayout.ItemHeight)
		lv.SetLayout(lv.Left, LVTop, lv.Width, lv.Height)
		Return
	End If
	
	' Tries to set the lv beneath the et, with the height required
	' to a minimum number of items (MinNumOfVisibleItems)
	If etBottom + MinNumOfVisibleItems * lv.SingleLineLayout.ItemHeight <= AvailableHeight Then
		lv.Height = Min(mLVMaxHeight, MinNumOfVisibleItems * lv.SingleLineLayout.ItemHeight)
		lv.SetLayout(lv.Left, LVTop, lv.Width, lv.Height)
		Return
	End If
	
	' --- Here the lv must be placed on top of et.
	' Tries to set height to the required height
	' or less, if the lv contains few items.
	If CurrentETTop <= RequiredHeight Then
		lv.Height = Min(mLVMaxHeight, Min(RequiredHeight, lv.Size * lv.SingleLineLayout.ItemHeight))
		LVTop = CurrentETTop - lv.Height
		lv.SetLayout(lv.Left, LVTop, lv.Width, lv.Height)
		Return
	End If
	
	' Set the height to a minimum number of items (3)
	If CurrentETTop - MinNumOfVisibleItems * lv.SingleLineLayout.ItemHeight >= 0 Then
		lv.Height = Min(mLVMaxHeight, MinNumOfVisibleItems * lv.SingleLineLayout.ItemHeight)
		LVTop = CurrentETTop - lv.Height
		lv.SetLayout(lv.Left, LVTop, lv.Width, lv.Height)
		Return
	End If
	
End Sub

Private Sub ResizeForKeyboard
	If KeyboardVisible Then
		If KeyboardTop < ETInitialTop Then
			ET.Top = KeyboardTop - ET.Height
			CurrentETTop = ET.Top
	'		et.SetLayout(et.Left, et.Top, et.Width, et.Height)
			ResizeListViewHeight
		End If
	Else
		ET.Top = ETInitialTop
		CurrentETTop = ET.Top
'		et.SetLayout(et.Left, et.Top, et.Width, et.Height)
		ResizeListViewHeight
	End If
End Sub